let jwt = require('jsonwebtoken');
let oauth2Server = require('oauth2-server');
let requestService = require('./request');
let responseService = require('./response');
let Request = oauth2Server.Request;
let Response = oauth2Server.Response;
let loginType;
let loginTypes = { oauth2: 'oauth2', jwt: 'jwt' };
let jwtUsers = [{ u: 'zd@zeedhi.com', p: 'zeedhi' }];
let secret = `it's secret`;

function AuthService(app) {

    /**
     * Checks user is authenticate
     * @param {Request} req Node request
     * @param {Response} res Node response
     * @param {Function} next Node next method
     * @returns {Promise.<Function>}
     */
    this.isAuthenticated = async (req, res, next) => {
        try {
            if (this.isOAuth2()) {
                await app.oauth.authenticate(new Request(req), new Response(res));
            } else if (this.isJWT()) {
                verifyJwt(req.headers.authorization);
            } else {
                throw false;
            }
            return next();
        } catch (err) {
            if (err && err.code) {
                responseService.error(res, err.status, err.code, err.message);
            } else {
                responseService.throwUnauthorized(res);
            }
        }
    };

    /**
     * Retrieves a random user from Randomuser API
     * @param {Object} [data={}] An object to merge with result
     * @returns {Promise.<Object>} A user
     */
    this.getRandomUser = async (data = {}) => {
        let body = await requestService.get('https://randomuser.me/api/?nat=br');
        return Object.assign(data, JSON.parse(body).results[0]);
    };

    /**
     * Difines login type as OAuth2
     */
    this.setLoginAsOAuth2 = () => {
        loginType = loginTypes.oauth2;
    };

    /**
     * Difines login type as JWT
     */
    this.setLoginAsJWT = () => {
        loginType = loginTypes.jwt;
    };

    /**
     * Checks if the login is OAuth2
     * @returns {Boolean}
     */
    this.isOAuth2 = () => {
        return loginType === loginTypes.oauth2;
    };

    /**
     * Checks if the login is JWT
     * @returns {Boolean}
     */
    this.isJWT = () => {
        return loginType === loginTypes.jwt;
    };

    /**
     * Checks if it's an authorized user
     * @param {String} username
     * @param {String} password
     * @returns {Boolean}
     */
    this.authUserJWT = (username, password) => {
        return !!jwtUsers.filter((user) => user.u === username && user.p === password).length;
    };

    /**
     * Retrieves sign secret
     * @returns {String}
     */
    this.getSecret = () => {
        return secret;
    };

    /**
     * Checks JWT is valid
     * @param {String} authorization Request bearer token
     */
    let verifyJwt = (authorization) => {
        if (!authorization) {
            throw false;
        }
        jwt.verify(authorization.split(' ')[1], secret);
    };
}

module = module.exports = AuthService;

